# -*- coding: utf-8 -*-
from django.contrib import admin
from .utils import get_model
from . import app_settings

Affiliate = get_model(app_settings.AFFILIATE_MODEL)


class AffiliateAdmin(admin.ModelAdmin):
    list_display = ('pk', 'user', 'is_active', 'reward_amount',
        'reward_percentage', 'created_at')


admin.site.register(Affiliate, AffiliateAdmin)



import logging

from . import app_settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.functional import SimpleLazyObject
from django.utils import timezone
from django.utils.dateparse import parse_datetime

from .models import NoAffiliate
from .utils import get_model

l = logging.getLogger(__name__)


AffiliateModel = get_model(app_settings.AFFILIATE_MODEL)

def get_affiliate(request, new_aid, prev_aid, prev_aid_dt):
    if not hasattr(request, '_cached_affiliate'):
        affiliate = AffiliateModel.objects.filter(pk=new_aid).first()
        if affiliate is None or not affiliate.is_active:
            prev_affiliate = None
            if prev_aid:
                prev_affiliate = AffiliateModel.objects.filter(pk=prev_aid).first()
            if prev_affiliate is None or not prev_affiliate.is_active:
                affiliate = affiliate or prev_affiliate or NoAffiliate()
            else:
                affiliate = prev_affiliate
                if app_settings.SAVE_IN_SESSION:
                    request.session['_aid'] = prev_aid
                    if prev_aid_dt:
                        request.session['_aid_dt'] = prev_aid_dt
        request._cached_affiliate = affiliate
    return request._cached_affiliate


class AffiliateMiddleware(object):

    def process_request(self, request):
        new_aid, prev_aid, prev_aid_dt = None, None, None
        if app_settings.SAVE_IN_SESSION:
            session = getattr(request, 'session', None)
            if not session:
                raise ImproperlyConfigured(
                    "session attribute should be set for request. Please add "
                    "'django.contrib.sessions.middleware.SessionMiddleware' "
                    "to your MIDDLEWARE_CLASSES")
            elif app_settings.SAVE_IN_SESSION:
                prev_aid = session.get('_aid', None)
                prev_aid_dt = session.get('_aid_dt', None)
        now = timezone.now()
        new_aid = request.GET.get(app_settings.PARAM_NAME, None)
        if new_aid:
            if app_settings.SAVE_IN_SESSION:
                session['_aid'] = new_aid
                session['_aid_dt'] = now.isoformat()
        if prev_aid and app_settings.SAVE_IN_SESSION:
            if prev_aid_dt is None:
                l.error('_aid_dt not found in session')
                if not new_aid:
                    session['_aid_dt'] = now.isoformat()
            else:
                prev_aid_dt_obj = parse_datetime(prev_aid_dt)
                if (now - prev_aid_dt_obj).total_seconds() > app_settings.SESSION_AGE:
                    # aid expired
                    prev_aid = None
                    prev_aid_dt = None
                    if not new_aid:
                        session.pop('_aid')
                        session.pop('_aid_dt')
        request.affiliate = SimpleLazyObject(lambda: get_affiliate(request, new_aid, prev_aid, prev_aid_dt))



# -*- coding: utf-8 -*-
from django.views import generic
from django.utils.translation import ugettext_lazy as _
from django.contrib.messages.views import SuccessMessageMixin
from django.contrib.auth.decorators import login_required
from .forms import AffiliateCreateForm
from .utils import get_affiliate_model


class LoginRequiredMixin(object):
    @classmethod
    def as_view(cls, **initkwargs):
        view = super(LoginRequiredMixin, cls).as_view(**initkwargs)
        return login_required(view)


class AffiliateBaseView(SuccessMessageMixin, LoginRequiredMixin, generic.CreateView):
    """
    Probably you need to redefine some methods here to respect
    your custom Affiliate model.
    """
    template_name = "partner/affiliate.html"
    form_class = AffiliateCreateForm

    def get_affiliate_banner_model(self):
        raise NotImplementedError()

    @property
    def affiliate(self):
        if not hasattr(self, '_affiliate'):
            self._affiliate = get_affiliate_model().objects.filter(user=self.request.user).first()
        return self._affiliate

    def get_form_kwargs(self):
        kwargs = super(AffiliateBaseView, self).get_form_kwargs()
        kwargs.update({'user': self.request.user})
        return kwargs

    def get_success_url(self):
        return self.request.get_full_path()

    def get_success_message(self, cleaned_data):
        return _("Affiliate account successfully created")

    def get_context_data(self, **kwargs):
        context = super(AffiliateBaseView, self).get_context_data(**kwargs)
        context['affiliate'] = self.affiliate
        if self.affiliate:
            context['affiliate_link'] = self.affiliate.build_absolute_affiliate_uri(self.request)
        return context



# -*- coding: utf-8 -*-
from django.utils.http import urlencode
from django.utils.six.moves.urllib.parse import parse_qsl, urlparse, urlunparse
from . import app_settings
try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model as django_get_model

    def get_model(app_label, model_name=None):
        if model_name is None:
            app_label, model_name = app_label.split('.')
        return django_get_model(app_label, model_name)


def get_affiliate_model():
    return get_model(app_settings.AFFILIATE_MODEL)


def add_affiliate_code(url, aid_code):
    parsed = urlparse(str(url))
    query = dict(parse_qsl(parsed.query))
    query.update({app_settings.PARAM_NAME: str(aid_code)})
    url_parts = list(parsed)
    url_parts[4] = urlencode(query)
    return urlunparse(url_parts)


def remove_affiliate_code(url):
    parsed = urlparse(str(url))
    query = dict(parse_qsl(parsed.query))
    query.pop(app_settings.PARAM_NAME, None)
    url_parts = list(parsed)
    url_parts[4] = urlencode(query)
    return urlunparse(url_parts)



from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings
from model_mommy import mommy

from affiliate.utils import get_affiliate_model

Affiliate = get_affiliate_model()


class TestAffiliateView(TestCase):
    def setUp(self):
        super(TestAffiliateView, self).setUp()
        self.user = mommy.make(settings.AUTH_USER_MODEL, username='john')
        self.user.set_password('123456')
        self.user.save()

    def test_partner_get(self):
        self.client.login(username=self.user.username, password='123456')
        resp = self.client.get(reverse('partner:affiliate'))
        self.assertEqual(resp.status_code, 200)

    def test_partner_create(self):
        self.client.login(username=self.user.username, password='123456')
        resp = self.client.post(reverse('partner:affiliate'))
        self.assertRedirects(resp, reverse('partner:affiliate'))
        affiliate = Affiliate.objects.get()
        self.assertEqual(affiliate.user, self.user)



# -*- coding: utf-8 -*-
from django import forms
from django.utils.translation import ugettext_lazy as _
from .utils import get_affiliate_model
Affiliate = get_affiliate_model()


class AffiliateCreateForm(forms.ModelForm):
    """
    Redefine it to respect your custom Affiliate model.
    This form asserts, that Affiliate has OneToOne relationship to User.
    """
    class Meta:
        model = Affiliate
        fields = ()

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(AffiliateCreateForm, self).__init__(*args, **kwargs)

    def clean(self):
        try:
            self.user.affiliate
            raise forms.ValidationError(_("Affiliate already created"))
        except Affiliate.DoesNotExist:
            pass
        return self.cleaned_data

    def save(self, commit=True):
        aff = Affiliate(user=self.user, **self.cleaned_data)
        if commit:
            aff.save()
        return aff



# -*- coding: utf-8 -*-
from django.conf import settings
from decimal import Decimal as D


AFFILIATE_MODEL = settings.AFFILIATE_AFFILIATE_MODEL  # mandatory, must be set
PARAM_NAME = getattr(settings, 'AFFILIATE_PARAM_NAME', 'aid')
REWARD_AMOUNT = getattr(settings, 'AFFILIATE_REWARD_AMOUNT', 10)
REWARD_PERCENTAGE = getattr(settings, 'AFFILIATE_REWARD_PERCENTAGE', True)
SAVE_IN_SESSION = getattr(settings, 'AFFILIATE_SAVE_IN_SESSION', True)
SESSION_AGE = getattr(settings, 'AFFILIATE_SESSION_AGE', 5 * 24 * 60 * 60)
DEFAULT_LINK = getattr(settings, 'AFFILIATE_DEFAULT_LINK', '/')
# deprecated
BANNER_FOLDER = getattr(settings, 'AFFILIATE_BANNER_PATH', 'affiliate')
START_AID = getattr(settings, 'AFFILIATE_START_AID', "1000")
DEFAULT_CURRENCY = getattr(settings, 'AFFILIATE_DEFAULT_CURRENCY', "USD")
MIN_REQUEST_AMOUNT = getattr(settings, 'AFFILIATE_MIN_BALANCE_FOR_REQUEST', D('1.0'))



from decimal import Decimal as D
from django.test import TestCase
from model_mommy import mommy

from affiliate.utils import get_affiliate_model

Affiliate = get_affiliate_model()


class TestAffiliateModel(TestCase):
    def test_build_partner_absolute_link(self):
        resp = self.client.get('/')
        request = resp.context['request']
        affiliate = mommy.make(Affiliate)
        url = affiliate.build_absolute_affiliate_uri(request, '/')
        self.assertEqual('http://testserver/?aid={0}'.format(affiliate.aid), url)

    def test_build_partner_relative_link(self):
        affiliate = mommy.make(Affiliate)
        url = affiliate.build_affiliate_url('/')
        self.assertEqual('/?aid={0}'.format(affiliate.aid), url)

    def test_calc_affiliate_reward_percent(self):
        affiliate = mommy.make(Affiliate, reward_amount=D('0.5'), reward_percentage=True)
        self.assertEqual(affiliate.calc_affiliate_reward(10000), D('50'))

    def test_calc_affiliate_reward_fixed(self):
        affiliate = mommy.make(Affiliate, reward_amount=D('0.5'), reward_percentage=False)
        self.assertEqual(affiliate.calc_affiliate_reward(D('0.5')), D('0.5'))



# -*- coding: utf-8 -*-
from datetime import timedelta

from django.test import TestCase
from django.utils import timezone
from django.core.urlresolvers import reverse
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from model_mommy import mommy
from freezegun import freeze_time

from affiliate import app_settings
from .utils import get_aid_url, modify_settings


class TestAffiliateMiddleware(TestCase):
    def test_no_affiliate_in_url(self):
        resp = self.client.get('/')
        self.assertEqual(resp.status_code, 200)
        self.assertFalse(resp.context['request'].affiliate.exists())

    def test_bad_affiliate_code(self):
        resp = self.client.get(get_aid_url('/', 123))
        self.assertEqual(resp.status_code, 200)
        self.assertFalse(resp.context['request'].affiliate.exists())

    def test_affiliate_assigned(self):
        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        resp = self.client.get(get_aid_url('/', affiliate.aid))
        self.assertEqual(resp.status_code, 200)
        affiliate_resp = resp.context['request'].affiliate
        self.assertTrue(affiliate_resp.exists())
        self.assertEqual(affiliate.aid, affiliate_resp.aid)

    def test_previous_affiliate_is_used(self):
        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        resp = self.client.get(get_aid_url('/', affiliate.aid))
        self.assertEqual(resp.status_code, 200)
        resp = self.client.get(get_aid_url('/', affiliate.aid + 100))  # invalid aid code
        self.assertEqual(resp.status_code, 200)
        affiliate_resp = resp.context['request'].affiliate
        self.assertTrue(affiliate_resp.exists())
        self.assertEqual(affiliate.aid, affiliate_resp.aid)

    def test_affiliate_saved_in_session(self):
        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        self.client.get(get_aid_url('/', affiliate.aid))

        # next response without aid still contains affiliate
        with freeze_time(timezone.now() + timedelta(seconds=app_settings.SESSION_AGE - 1)):
            resp = self.client.get('/')

        self.assertEqual(resp.status_code, 200)
        affiliate_resp = resp.context['request'].affiliate
        self.assertTrue(affiliate_resp.exists())
        self.assertEqual(affiliate.aid, affiliate_resp.aid)

    def test_affiliate_session_expired(self):
        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        self.client.get(get_aid_url('/', affiliate.aid))

        # affiliate is expired
        with freeze_time(timezone.now() + timedelta(seconds=app_settings.SESSION_AGE + 1)):
            resp = self.client.get('/')

        self.assertEqual(resp.status_code, 200)
        self.assertFalse(resp.context['request'].affiliate.exists())

    def test_previous_affiliate_session_expired(self):
        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        resp = self.client.get(get_aid_url('/', affiliate.aid))
        self.assertEqual(resp.status_code, 200)

        # affiliate is expired
        with freeze_time(timezone.now() + timedelta(seconds=app_settings.SESSION_AGE + 1)):
            resp = self.client.get(get_aid_url('/', affiliate.aid + 100))  # invalid aid code

        self.assertEqual(resp.status_code, 200)
        self.assertFalse(resp.context['request'].affiliate.exists())

    def test_affiliate_code_in_post_request(self):
        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        resp = self.client.post(get_aid_url(reverse('users:signup'), affiliate.aid),
            dict(username='newuser', password1='123456', password2='123456'))
        self.assertEqual(resp.status_code, 302)
        resp = self.client.get('/')
        self.assertTrue(resp.context['request'].affiliate.exists())
        self.assertEqual(resp.context['request'].affiliate.pk, affiliate.pk)


@modify_settings(MIDDLEWARE_CLASSES={
    'remove': [
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ],
}, INSTALLED_APPS={
    'remove': [
        'django.contrib.sessions'
    ]
})
class TestAffiliateMiddlewareNoSession(TestCase):
    def test_no_session_affiliate_in_url(self):
        app_settings.SAVE_IN_SESSION = False

        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        resp = self.client.get(get_aid_url('/', affiliate.aid))
        self.assertEqual(resp.status_code, 200)
        affiliate_resp = resp.context['request'].affiliate
        self.assertTrue(affiliate_resp.exists())
        self.assertEqual(affiliate.aid, affiliate_resp.aid)

        # next request can't remember previous affiliate
        resp = self.client.get('/')
        self.assertEqual(resp.status_code, 200)
        self.assertFalse(resp.context['request'].affiliate.exists())

    def test_no_session_exception_raised(self):
        app_settings.SAVE_IN_SESSION = True

        affiliate = mommy.make(settings.AFFILIATE_AFFILIATE_MODEL)
        with self.assertRaises(ImproperlyConfigured):
            self.client.get(get_aid_url('/', affiliate.aid))



from __future__ import print_function, division, absolute_import, unicode_literals
import logging
from decimal import Decimal as D
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import six
from django.conf import settings
from . import app_settings
from .utils import add_affiliate_code

l = logging.getLogger(__name__)


class AffiliateManager(models.Manager):
    def create_affiliate(self, user, **extra_fields):
        affiliate = self.model(user=user, **extra_fields)
        affiliate.save()
        return affiliate


class AbstractAffiliate(models.Model):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, verbose_name=_("user"))
    reward_amount = models.DecimalField(_("reward amount"), max_digits=16,
        decimal_places=5, default=app_settings.REWARD_AMOUNT)
    reward_percentage = models.BooleanField(_('in percent'),
        default=app_settings.REWARD_PERCENTAGE)
    is_active = models.BooleanField(_('active'), default=True)
    created_at = models.DateTimeField(_("created at"), auto_now_add=True)

    objects = AffiliateManager()

    class Meta:
        abstract = True
        verbose_name = _("affiliate")
        verbose_name_plural = _("affiliates")

    def __str__(self):
        return six.text_type(self.aid)

    @property
    def aid(self):
        return self.pk

    def exists(self):
        return True

    def build_absolute_affiliate_uri(self, request, location=None):
        location = location or app_settings.DEFAULT_LINK
        uri = request.build_absolute_uri(location)
        return add_affiliate_code(uri, self.pk)

    def build_affiliate_url(self, location=None):
        location = location or app_settings.DEFAULT_LINK
        return add_affiliate_code(location, self.pk)

    def calc_affiliate_reward(self, total_price):
        if self.reward_percentage:
            return total_price * (self.reward_amount / D('100'))
        else:
            return self.reward_amount

    @property
    def quantized_reward_amount(self):
        return self.reward_amount.quantize(D('0.01'))


class NoAffiliate(object):
    is_active = False

    def exists(self):
        return False
